require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xwebapplication',
  dscmeta_resource_friendly_name: 'xWebApplication',
  dscmeta_resource_name: 'MSFT_xWebApplication',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '1.11.0.0',
  docs: 'The DSC xWebApplication resource type.
         Automatically generated from version 1.11.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_physicalpath: {
      type: 'String',
      desc: 'Physical path for the web application directory',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_website: {
      type: 'String',
      desc: 'Name of website with which web application is associated',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_authenticationinfo: {
      type: 'Optional[Struct[{
              basic => Optional[Boolean],
              anonymous => Optional[Boolean],
              digest => Optional[Boolean],
              windows => Optional[Boolean]
            }]]',
      desc: 'Hashtable containing authentication information (Anonymous, Basic, Digest, Windows)',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_xWebApplicationAuthenticationInformation',
      mof_is_embedded: true,
    },
    dsc_preloadenabled: {
      type: 'Optional[Boolean]',
      desc: 'Allows a Application to automatically start without a request',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_serviceautostartprovider: {
      type: 'Optional[String]',
      desc: 'Adds a AutostartProvider',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sslflags: {
      type: "Optional[Array[Enum['Ssl', 'SslNegotiateCert', 'SslRequireCert']]]",
      desc: 'SSLFlags for the application',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of web application',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_applicationtype: {
      type: 'Optional[String]',
      desc: 'Adds a AutostartProvider ApplicationType',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_webapppool: {
      type: 'String',
      desc: 'Web application pool for the web application',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_serviceautostartenabled: {
      type: 'Optional[Boolean]',
      desc: 'Enables Autostart on a Application.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'Whether web application should be present or absent',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
