require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xwebsite',
  dscmeta_resource_friendly_name: 'xWebsite',
  dscmeta_resource_name: 'MSFT_xWebsite',
  dscmeta_module_name: 'xWebAdministration',
  dscmeta_module_version: '1.11.0.0',
  docs: 'The DSC xWebsite resource type.
         Automatically generated from version 1.11.0.0',
  features: ['simple_get_filter', 'canonicalize'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    dsc_name: {
      type: 'String',
      desc: ' ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_preloadenabled: {
      type: 'Optional[Boolean]',
      desc: 'Allows the Website to automatically start without a request',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_serviceautostartenabled: {
      type: 'Optional[Boolean]',
      desc: 'Enables Autostart on a Website.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_bindinginfo: {
      type: "Optional[Array[Struct[{
              sslflags => Optional[Enum['0', '1', '2', '3']],
              certificatestorename => Optional[Enum['My', 'WebHosting']],
              certificatethumbprint => Optional[String],
              hostname => Optional[String],
              bindinginformation => Optional[String],
              port => Optional[Integer[0, 65535]],
              ipaddress => Optional[String],
              protocol => Enum['http', 'https', 'msmq.formatname', 'net.msmq', 'net.pipe', 'net.tcp']
            }]]]",
      desc: "Website's binding information in the form of an array of embedded instances of the MSFT_xWebBindingInformation CIM class.",

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_xWebBindingInformation[]',
      mof_is_embedded: true,
    },
    dsc_applicationpool: {
      type: 'Optional[String]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_physicalpath: {
      type: 'Optional[String]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_defaultpage: {
      type: 'Optional[Array[String]]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String[]',
      mof_is_embedded: false,
    },
    dsc_state: {
      type: "Optional[Enum['Started', 'Stopped']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_applicationtype: {
      type: 'Optional[String]',
      desc: 'Adds a AutostartProvider ApplicationType',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_serviceautostartprovider: {
      type: 'Optional[String]',
      desc: 'Adds a AutostartProvider',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enabledprotocols: {
      type: 'Optional[String]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_authenticationinfo: {
      type: 'Optional[Struct[{
              basic => Optional[Boolean],
              anonymous => Optional[Boolean],
              digest => Optional[Boolean],
              windows => Optional[Boolean]
            }]]',
      desc: 'Hashtable containing authentication information (Anonymous, Basic, Digest, Windows)',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'MSFT_xWebAuthenticationInformation',
      mof_is_embedded: true,
    },
  },
)
